@@ -139,9 +139,7 @@ class _sha3.shake_256 "SHA3object *" "&SHAKE256type"
 typedef struct {
     PyObject_HEAD
     SHA3_state hash_state;
-#ifdef WITH_THREAD
     PyThread_type_lock lock;
-#endif
 } SHA3object;
 
 static PyTypeObject SHA3_224type;
@@ -167,9 +165,7 @@ newSHA3object(PyTypeObject *type)
     if (newobj == NULL) {
         return NULL;
     }
-#ifdef WITH_THREAD
     newobj->lock = NULL;
-#endif
     return newobj;
 }
 
@@ -182,10 +178,10 @@ py_sha3_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
     HashReturn res;
     PyObject *data = NULL;
 
-    if (!_PyArg_NoKeywords(type->tp_name, kwargs)) {
+    if (!_PyArg_NoKeywords(_PyType_Name(type), kwargs)) {
         return NULL;
     }
-    if (!PyArg_UnpackTuple(args, type->tp_name, 0, 1, &data)) {
+    if (!PyArg_UnpackTuple(args, _PyType_Name(type), 0, 1, &data)) {
         return NULL;
     }
 
@@ -223,7 +219,6 @@ py_sha3_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 
     if (data) {
         GET_BUFFER_VIEW_OR_ERROR(data, &buf, goto error);
-#ifdef WITH_THREAD
         if (buf.len >= HASHLIB_GIL_MINSIZE) {
             /* invariant: New objects can't be accessed by other code yet,
              * thus it's safe to release the GIL without locking the object.
@@ -235,9 +230,6 @@ py_sha3_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
         else {
             res = SHA3_process(&self->hash_state, buf.buf, buf.len * 8);
         }
-#else
-        res = SHA3_process(&self->hash_state, buf.buf, buf.len * 8);
-#endif
         if (res != SUCCESS) {
             PyErr_SetString(PyExc_RuntimeError,
                             "internal error in SHA3 Update()");
@@ -264,11 +256,9 @@ py_sha3_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 static void
 SHA3_dealloc(SHA3object *self)
 {
-#ifdef WITH_THREAD
     if (self->lock) {
         PyThread_free_lock(self->lock);
     }
-#endif
     PyObject_Del(self);
 }
 
@@ -372,7 +362,6 @@ _sha3_sha3_224_update(SHA3object *self, PyObject *data)
     GET_BUFFER_VIEW_OR_ERROUT(data, &buf);
 
     /* add new data, the function takes the length in bits not bytes */
-#ifdef WITH_THREAD
     if (self->lock == NULL && buf.len >= HASHLIB_GIL_MINSIZE) {
         self->lock = PyThread_allocate_lock();
     }
@@ -390,9 +379,6 @@ _sha3_sha3_224_update(SHA3object *self, PyObject *data)
     else {
         res = SHA3_process(&self->hash_state, buf.buf, buf.len * 8);
     }
-#else
-    res = SHA3_process(&self->hash_state, buf.buf, buf.len * 8);
-#endif
 
     if (res != SUCCESS) {
         PyBuffer_Release(&buf);
@@ -402,8 +388,7 @@ _sha3_sha3_224_update(SHA3object *self, PyObject *data)
     }
 
     PyBuffer_Release(&buf);
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 
@@ -508,10 +493,10 @@ static PyGetSetDef SHA3_getseters[] = {
         0,                  /* tp_itemsize */ \
         /*  methods  */ \
         (destructor)SHA3_dealloc, /* tp_dealloc */ \
-        0,                  /* tp_print */ \
+        0,                  /* tp_vectorcall_offset */ \
         0,                  /* tp_getattr */ \
         0,                  /* tp_setattr */ \
-        0,                  /* tp_reserved */ \
+        0,                  /* tp_as_async */ \
         0,                  /* tp_repr */ \
         0,                  /* tp_as_number */ \
         0,                  /* tp_as_sequence */ \
@@ -597,18 +582,13 @@ SHA3_TYPE(Keccak_512type, "_sha3.keccak_512", keccak_512__doc__, SHA3_methods);
 
 
 static PyObject *
-_SHAKE_digest(SHA3object *self, PyObject *digestlen_obj, int hex)
+_SHAKE_digest(SHA3object *self, unsigned long digestlen, int hex)
 {
-    unsigned long digestlen;
     unsigned char *digest = NULL;
     SHA3_state temp;
     int res;
     PyObject *result = NULL;
 
-    digestlen = PyLong_AsUnsignedLong(digestlen_obj);
-    if (digestlen == (unsigned long) -1 && PyErr_Occurred()) {
-        return NULL;
-    }
     if (digestlen >= (1 << 29)) {
         PyErr_SetString(PyExc_ValueError, "length is too large");
         return NULL;
@@ -652,15 +632,15 @@ _SHAKE_digest(SHA3object *self, PyObject *digestlen_obj, int hex)
 /*[clinic input]
 _sha3.shake_128.digest
 
-    length: object
+    length: unsigned_long
     /
 
 Return the digest value as a bytes object.
 [clinic start generated code]*/
 
 static PyObject *
-_sha3_shake_128_digest(SHA3object *self, PyObject *length)
-/*[clinic end generated code: output=eaa80b6299142396 input=c579eb109f6227d2]*/
+_sha3_shake_128_digest_impl(SHA3object *self, unsigned long length)
+/*[clinic end generated code: output=2313605e2f87bb8f input=418ef6a36d2e6082]*/
 {
     return _SHAKE_digest(self, length, 0);
 }
@@ -669,15 +649,15 @@ _sha3_shake_128_digest(SHA3object *self, PyObject *length)
 /*[clinic input]
 _sha3.shake_128.hexdigest
 
-    length: object
+    length: unsigned_long
     /
 
 Return the digest value as a string of hexadecimal digits.
 [clinic start generated code]*/
 
 static PyObject *
-_sha3_shake_128_hexdigest(SHA3object *self, PyObject *length)
-/*[clinic end generated code: output=4752f90e53c8bf2a input=a82694ea83865f5a]*/
+_sha3_shake_128_hexdigest_impl(SHA3object *self, unsigned long length)
+/*[clinic end generated code: output=bf8e2f1e490944a8 input=69fb29b0926ae321]*/
 {
     return _SHAKE_digest(self, length, 1);
 }
